<?php

namespace flipbox\hubspot\modules\resources\services;

use craft\helpers\Json;
use flipbox\hubspot\authentication\AuthenticationStrategyInterface;
use flipbox\hubspot\HubSpot;
use Flipbox\Transform\Factory;
use Flipbox\Transform\Transformers\TransformerInterface;

class TimelineEvents extends AbstractResource
{

    /**
     * @param $data
     * @param callable|TransformerInterface        $transformer
     * @param AuthenticationStrategyInterface|null $authenticationStrategy
     * @return array
     */
    public function create(
        $appId,
        $data,
        callable $transformer,
        AuthenticationStrategyInterface $authenticationStrategy = null
    ) {
        $payload = Factory::item($transformer, $data);

        $response = HubSpot::getInstance()->http()->timelineEvents()->create(
            $appId,
            $payload,
            $authenticationStrategy
        );

        // Interpret response
        if ($response->getStatusCode() !== 200) {
            $body = Json::decodeIfJson($response->getBody()->getContents());

            HubSpot::warning(
                sprintf(
                    "Unable to push timeline event: %s, errors: %s",
                    Json::encode($payload),
                    Json::encode($body)
                )
            );

            HubSpot::warning(
               'STATUS CODE: ' . $response->getStatusCode()
            );
            HubSpot::warning(
                Json::decodeIfJson($response->getBody()->getContents())
            );
            return [
                false,
                $body
            ];
        }

        return [
            true,
            Json::decodeIfJson($response->getBody()->getContents())
        ];
    }

}
